package com.sybase.sup.samples.objectapi;

import net.rim.device.api.system.Characters;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.Graphics;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.UiApplication;
import net.rim.device.api.ui.component.BasicEditField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.GaugeField;
import net.rim.device.api.ui.component.KeywordFilterField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.MainScreen;
import SUP101.Customer;
import SUP101.SUP101DB;

import com.sybase.persistence.ObjectSyncStatusData;
import com.sybase.persistence.SyncStatusListener;
import com.sybase.persistence.SyncStatusState;

/**
 * @author bdeng
 * 
 *         This class represents the main screen for the Customer Sample
 *         application.
 * 
 *         Note: For the sake of simplicity, this sample application may not
 *         leverage resource bundles and resource strings. However, it is
 *         STRONGLY recommended that application developers make use of the
 *         localization features available within the BlackBerry development
 *         platform to ensure a seamless application experience across a variety
 *         of languages and geographies. For more information on localizing your
 *         application, please refer to the BlackBerry Java Development
 *         Environment Development Guide.
 */
public final class CustomerSampleScreen extends MainScreen
        implements SyncStatusListener
{
    private CustomerSample     _app;
    private KeywordFilterField _keywordFilterField;
    // last sync status
    private int                _lastSyncState = -1;
    // synchronization indicator
    private GaugeField         _syncProgressField;
    // if a synchronization is happening so that we display an indicator
    private boolean            _isSynchronizing;
    // whether there are pending changes
    private boolean            _changed;

    /**
     * Creates a new CustomerSampleScreen.
     * 
     * @param app
     *            The UiApplication creating an instance of this class.
     */
    public CustomerSampleScreen(CustomerSample app)
    {
        // A reference to the UiApplication instance for use in this class.
        _app = app;
        // We need a reference to the UiApplication's KeywordFilterField.
        _keywordFilterField = _app.getKeywordFilterField();
        // Add menu item to the screen's menu.
        addMenuItem(syncItem);
    }

    /**
     * Intercepts the ENTER key and displays info screen.
     * 
     * @see net.rim.device.api.ui.Screen#keyChar(char,int,int)
     * */
    protected boolean keyChar(char key, int status, int time)
    {
        if ( key == Characters.ENTER )
        {
            displayInfoScreen();
            return true; // We've consumed the event.
        }
        return super.keyChar(key, status, time);
    }

    /**
     * Handles a trackball click.
     * 
     * @see net.rim.device.api.ui.Screen#invokeAction(int)
     */
    public boolean invokeAction(int action)
    {
        switch (action)
        {
            case ACTION_INVOKE: // Trackball click.
                displayInfoScreen();
                return true; // We've consumed the event.
        }
        return super.invokeAction(action);
    }

    /**
     * Creates an InfoScreen instance and pushes it onto the stack for
     * rendering.
     */
    private void displayInfoScreen()
    {
        // Retrieve the selected Customer and use it to invoke a new InfoScreen.
        Customer customer = (Customer) _keywordFilterField.getSelectedElement();
        if ( customer != null )
        {
            InfoScreen infoScreen = new InfoScreen(this, customer);
            _app.pushScreen(infoScreen);
        }
    }

    /**
     * Prevent the save dialog from being displayed.
     * 
     * @see net.rim.device.api.ui.container.MainScreen#onSavePrompt()
     */
    public boolean onSavePrompt()
    {
        return true;
    }

    // Inner classes------------------------------------------------------------
    /**
     * Synchronizes the mbos
     */
    private final MenuItem syncItem = new MenuItem("Synchronize", 0, 0)
                                    {
                                        public void run()
                                        {
                                            synchronize();
                                        }
                                    };

    /**
     * A MainScreen class to display/update secondary information for a selected
     * customer.
     */
    private final static class InfoScreen extends MainScreen
    {
        private CustomerSampleScreen _sampleScreen;
        private Customer             _customer;
        private BasicEditField       _fnameField;
        private BasicEditField       _lnameField;
        private BasicEditField       _companyField;
        private BasicEditField       _addressField;
        private BasicEditField       _stateField;
        private BasicEditField       _cityField;
        private BasicEditField       _phoneField;
        private BasicEditField       _zipField;

        /**
         * Constructs a screen to display
         * 
         * @param customer
         *            The customer to display secondary information about
         */
        InfoScreen(CustomerSampleScreen sampleScreen, Customer customer)
        {
            _sampleScreen = sampleScreen;
            _customer = customer;
            // Set up and display UI elements.
            setTitle("Update Customer");
            _fnameField = new BasicEditField("First name: ", customer.getFname(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _lnameField = new BasicEditField("Last name: ", customer.getLname(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _companyField = new BasicEditField("Company: ", customer.getCompany_name(),
                    BasicEditField.DEFAULT_MAXCHARS, Field.FOCUSABLE);
            _addressField = new BasicEditField("Address: ", customer.getAddress(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _stateField = new BasicEditField("State: ", customer.getState(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _cityField = new BasicEditField("City: ", customer.getCity(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _phoneField = new BasicEditField("Phone: ", customer.getPhone(), BasicEditField.DEFAULT_MAXCHARS,
                    Field.FOCUSABLE);
            _zipField = new BasicEditField("Zip: ", customer.getZip(), BasicEditField.DEFAULT_MAXCHARS, Field.FOCUSABLE);
            add(_fnameField);
            add(_lnameField);
            add(_companyField);
            add(_addressField);
            add(_stateField);
            add(_cityField);
            add(_phoneField);
            add(_zipField);
            HorizontalFieldManager hfm = new HorizontalFieldManager(Field.FIELD_HCENTER);
            ButtonField submitButton = new ButtonField("submit", Field.FIELD_RIGHT)
            {
                protected boolean navigationClick(int status, int time)
                {
                    submit();
                    return true;
                }
            };
            ButtonField cancelButton = new ButtonField("cancel", Field.FIELD_LEFT)
            {
                protected boolean navigationClick(int status, int time)
                {
                    close();
                    return true;
                }
            };
            hfm.add(submitButton);
            hfm.add(cancelButton);
            add(hfm);
        }

        protected void submit()
        {
            _customer.setFname(_fnameField.getText().trim());
            _customer.setLname(_lnameField.getText().trim());
            _customer.setCompany_name(_companyField.getText().trim());
            _customer.setAddress(_addressField.getText().trim());
            _customer.setState(_stateField.getText().trim());
            _customer.setCity(_cityField.getText().trim());
            _customer.setPhone(_phoneField.getText().trim());
            _customer.setZip(_zipField.getText().trim());
            _customer.save();
            _sampleScreen._changed = true;
            _sampleScreen._keywordFilterField.updateList();
            close();
        }
    }

    // synchronizes the mbo
    private void synchronize()
    {
        Runnable runnable = new Runnable()
        {
            public void run()
            {
                synchronized (_app.getSyncLock())
                {
                    setSynchronizing(true);
                    try
                    {
                        // login to sync. After initial successful login, subsequent logins are invoked off line.
                        SUP101DB.loginToSync("supAdmin", "s3pAdmin");
                        if ( _changed )
                        {
                            _changed = false;
                            SUP101DB.submitPendingOperations();
                        }
                        SUP101DB.synchronize(CustomerSampleScreen.this);
                    }
                    catch (Exception e)
                    {
                    }
                    finally
                    {
                        setSynchronizing(false);
                    }
                }
            }
        };
        new Thread(runnable).start();
    }

    /**
     * Sets sync indicator and updates the sync status message.
     * 
     * @param isSynchronizing
     *            whether synchronization is happending
     */
    public void setSynchronizing(boolean isSynchronizing)
    {
        this._isSynchronizing = isSynchronizing;
        if ( this._isSynchronizing )
        {
            _syncProgressField = new GaugeField("", SyncStatusState.SYNC_STARTING, SyncStatusState.SYNC_DONE, 0,
                    GaugeField.NO_TEXT | GaugeField.LABEL_AS_PROGRESS)
            {
                protected void paint(Graphics g)
                {
                    super.paint(g);
                    int indicatorHeigth = 20;
                    int fontSize = 12;
                    int y = getHeight() - indicatorHeigth;
                    // setting up font style
                    Font f = Font.getDefault().derive(Font.PLAIN, fontSize);
                    g.setFont(f);
                    g.setColor(0x00000000);
                    g.setDrawingStyle(Graphics.DRAWSTYLE_AALINES, true);
                    // paint the synchronizing indicator as it can't be
                    // displayed in the progress bar using
                    // LABEL_AS_PROGRESS style
                    g.drawText(getLabel(), 1, y + (indicatorHeigth - fontSize) / 2);
                    g.setColor(0x00BEBEBE);
                    g.drawRect(0, 0, getWidth(), getHeight());
                }
            };
            _app.invokeLater(new Runnable()
            {
                public void run()
                {
                    setStatus(_syncProgressField);
                }
            });
        }
        else
        {
            _app.invokeLater(new Runnable()
            {
                public void run()
                {
                    _syncProgressField = null;
                    setStatus(null);
                }
            });
            _app.refreshData();
        }
    }

    /**
     * Called when there is information to report about the status of a
     * synchronization that is taking place.
     * 
     * @param data
     *            A {@link ObjectSyncStatusData} object that contains
     *            information about the status of the running synchronization.
     * @return Return true to cancel the synchronization or false to continue.
     */
    public boolean objectSyncStatus(ObjectSyncStatusData data)
    {
        if ( !_isSynchronizing )
        {
            return false;
        }
        final int syncState = data.getSyncStatusState();
        if ( _lastSyncState != syncState )
        {
            _lastSyncState = syncState;
            String syncMsg = null;
            if ( syncState == SyncStatusState.SYNC_STARTING )
            {
                syncMsg = "Connecting...";
            }
            else if ( syncState == SyncStatusState.APPLICATION_DATA_UPLOADING )
            {
                syncMsg = "Metadata Uploading...";
            }
            else if ( syncState == SyncStatusState.APPLICATION_DATA_DOWNLOADING )
            {
                syncMsg = "Metadata Downloading...";
            }
            else if ( syncState == SyncStatusState.SYNC_DONE )
            {
                syncMsg = "Completed!";
            }
            if ( syncMsg != null )
            {
                final StringBuffer strLabel = new StringBuffer(256);
                strLabel.append("Retrieving synchronizationgroup:").append("default").append(" - ").append(syncMsg);
                UiApplication.getUiApplication().invokeLater(new Runnable()
                {
                    public void run()
                    {
                        _syncProgressField.setValue(syncState);
                        _syncProgressField.setLabel(strLabel.toString());
                    }
                });
            }
        }
        return false;
    }
}
